@@ -1,7 +1,7 @@
 #########################################################################################
 # Package        HiPi::Device::GPIO
 # Description:   Wrapper for GPIO
-# Copyright    : Copyright (c) 2013-2017 Mark Dootson
+# Copyright    : Copyright (c) 2013-2023 Mark Dootson
 # License      : This is free software; you can redistribute it and/or modify it under
 #                the same terms as the Perl 5 programming language system itself.
 #########################################################################################
@@ -17,7 +17,7 @@ use HiPi qw( :rpi );
 use HiPi::Device::GPIO::Pin;
 use Fcntl;
 
-our $VERSION ='0.81';
+our $VERSION ='0.89';
 
 my $sysroot = '/sys/class/gpio';
 
@@ -210,6 +210,26 @@ sub get_pin_interrupt {
     return $edge;
 }
 
+sub set_pin_schmitt {
+    warn q(HiPi::Device::GPIO does not support schmitt actions);
+    return undef;
+}
+
+sub get_pin_schmitt {
+    warn q(HiPi::Device::GPIO does not support schmitt actions);
+    return undef;
+}
+
+sub set_pin_slew {
+    warn q(HiPi::Device::GPIO does not support slew actions);
+    return undef;
+}
+
+sub get_pin_slew {
+    warn q(HiPi::Device::GPIO does not support slew actions);
+    return undef;
+}
+
 sub _do_export {
     my ($class, $pinno ) = @_;
     my $pinroot = qq(${sysroot}/gpio${pinno});
