@@ -2,7 +2,7 @@
 # Package        HiPi::Device::GPIO::Pin
 # Description:   Pin
 # Created        Wed Feb 20 04:37:38 2013
-# Copyright    : Copyright (c) 2013-2017 Mark Dootson
+# Copyright    : Copyright (c) 2013-2013 Mark Dootson
 # License      : This is free software; you can redistribute it and/or modify it under
 #                the same terms as the Perl 5 programming language system itself.
 #########################################################################################
@@ -17,7 +17,7 @@ use Carp;
 use Fcntl;
 use HiPi qw( :rpi );
 
-our $VERSION ='0.81';
+our $VERSION ='0.89';
 
 __PACKAGE__->create_accessors();
 
@@ -82,6 +82,26 @@ sub _do_getpud {
     return HiPi::Device::GPIO->get_pin_pud($self->pinid);
 }
 
+sub _do_setschmitt {
+    my( $self, $val ) = @_;
+    return HiPi::Device::GPIO->set_pin_schmitt($self->pinid, $val);
+}
+
+sub _do_getschmitt {
+    my( $self ) = @_;
+    return HiPi::Device::GPIO->get_pin_schmitt($self->pinid);
+}
+
+sub _do_setslew {
+    my( $self, $val ) = @_;
+    return HiPi::Device::GPIO->set_pin_slew($self->pinid, $val);    
+}
+
+sub _do_getslew {
+    my( $self ) = @_;
+    return HiPi::Device::GPIO->get_pin_slew($self->pinid);
+}
+
 sub _do_activelow {
     my($self, $newval) = @_;
     
