@@ -1,7 +1,7 @@
 #########################################################################################
 # Package        HiPi::Interface::HopeRF69
 # Description  : Control Hope RF69 Transceivers
-# Copyright    : Copyright (c) 2013-2020 Mark Dootson
+# Copyright    : Copyright (c) 2013-2023 Mark Dootson
 # License      : This is free software; you can redistribute it and/or modify it under
 #                the same terms as the Perl 5 programming language system itself.
 #########################################################################################
@@ -16,11 +16,23 @@ use parent qw( HiPi::Interface );
 use Carp;
 use HiPi qw( :rpi :spi :hrf69 );
 
-__PACKAGE__->create_accessors( qw( devicename reset_gpio update_default_on_reset
-                                   fsk_config ook_config ook_repeat
-                                   high_power_module max_power_on ) );
+__PACKAGE__->create_accessors( qw(
+    devicename
+    reset_gpio
+    update_default_on_reset
+    fsk_config
+    ook_config
+    ook_repeat
+) );
 
-our $VERSION ='0.82';
+__PACKAGE__->create_ro_accessors ( qw (
+    high_power_module
+    transmit_dbm
+    pa_boost_on
+    pa1_limit
+) );
+
+our $VERSION ='0.89';
 
 # Hope recommended updated reset defaults
 my $reset_defaults = [
@@ -39,13 +51,17 @@ sub new {
     
     my %params = (
         devicename   => '/dev/spidev0.1',
-        speed        => 9600000,  # 9.6 mhz
+        speed        => 8000000,  # 8 mhz
         bitsperword  => 8,
         delay        => 0,
         device       => undef,
         reset_gpio   => undef,
         update_default_on_reset => 1,
-        ook_repeat   => 15,
+        ook_repeat   => 25,
+        high_power_module => 0,
+        transmit_dbm => 10,
+        pa1_limit    => 10,
+           
         fsk_config => [
             [ RF69_REG_REGDATAMODUL,  0x00 ],   # modulation scheme FSK
             [ RF69_REG_FDEVMSB, 	  0x01 ],   # frequency deviation 5kHz 0x0052 -> 30kHz 0x01EC
@@ -58,12 +74,12 @@ sub new {
             [ RF69_REG_PREAMBLELSB,   0x03 ],   # 3 byte preamble
             [ RF69_REG_LNA, 		  0x08 ],	# 200ohms, gain by AGC loop -> 50ohms
             [ RF69_REG_RXBW, 		  0x43 ],	# channel filter bandwidth 10kHz -> 60kHz  page:26
-            [ RF69_REG_BITRATEMSB, 	  0x1A ],	# 4800b/s
-            [ RF69_REG_BITRATELSB, 	  0x0B ],	# 4800b/s
+            [ RF69_REG_BITRATEMSB, 	  RF69_VAL_BITRATEMSB_4800 ],	# 4800b/s
+            [ RF69_REG_BITRATELSB, 	  RF69_VAL_BITRATELSB_4800 ],	# 4800b/s
             [ RF69_REG_SYNCCONFIG, 	  0x88 ],	# Size of the Synch word = 2 (SyncSize + 1)
             [ RF69_REG_SYNCVALUE1, 	  0x2D ],	# 1st byte of Sync word
             [ RF69_REG_SYNCVALUE2, 	  0xD4 ],	# 2nd byte of Sync word
-            [ RF69_REG_PACKETCONFIG1, 0xA0 ],   # Variable length, Manchester coding
+            [ RF69_REG_PACKETCONFIG1, 0xA0 ],   # Variable length, Manchester coding, No CRC, No Address filtering
             [ RF69_REG_PAYLOADLEN, 	  0x42 ],	# max Length in RX, not used in Tx
             [ RF69_REG_NODEADDRESS,   0x06 ],	# Node address used in address filtering ( not used in this config )
             [ RF69_REG_FIFOTHRESH, 	  0x81 ],	# Condition to start packet transmission: at least one byte in FIFO
@@ -95,7 +111,7 @@ sub new {
     );
     
     foreach my $key (sort keys(%userparams)) {
-        $params{$key} = $userparams{$key};
+        $params{$key} = $userparams{$key} if(defined($userparams{$key}));
     }
     
     unless( defined($params{device}) ) {
@@ -109,27 +125,111 @@ sub new {
         );
     }
     
+    ## adjust allowed dBm for module type
+    
+    my $dbm = $params{transmit_dbm};
+    
+    $dbm //= 10;
+    $dbm = int($dbm);
+    
+    $params{pa_boost_on} = 0;
+    $params{high_power_module} = ( $params{high_power_module} ) ? 1 : 0;
+    if ( $params{high_power_module} ) {
+        $dbm = -2 if $dbm < -2;
+        $dbm = 20 if $dbm > 20;
+        $params{pa_boost_on} = 1 if ($dbm >= 18 );
+        $params{pa1_limit} = 2 if( $params{pa1_limit} < 2 );
+        $params{pa1_limit} = 13 if( $params{pa1_limit} > 13);
+    } else {
+        $dbm = -18 if $dbm < -18;
+        $dbm = 13 if $dbm > 13;
+    }
+    
+    $params{transmit_dbm} = $dbm;
+
     my $self = $class->SUPER::new(%params);
-    
-    # setup defaults
-    $self->reset();
-    
-    $self->configure($self->fsk_config);
+        
+    $self->init_radio_module;
     
     return $self;
 }
 
+sub init_radio_module {
+    my $self = shift;
+    
+    my $dbm = $self->transmit_dbm;
+    
+    my $powerlevel;
+    
+    if ( $self->high_power_module ) {
+        if ( $dbm <= $self->pa1_limit ) {
+            # PA1
+            my $reglsbits = $dbm + 18;
+            $powerlevel = RF69_PALEVEL_PA1_ON | ( $reglsbits & 0x1F );
+        } elsif( $dbm <= 17 ) {
+            # PA1 + PA2
+            my $reglsbits = $dbm + 14;
+            $powerlevel = RF69_PALEVEL_PA1_ON | RF69_PALEVEL_PA2_ON | ( $reglsbits & 0x1F );
+        } else {
+            # PA1 + PA2 + PA BOOST
+            my $reglsbits = $dbm + 11;
+            $powerlevel = RF69_PALEVEL_PA1_ON | RF69_PALEVEL_PA2_ON | ( $reglsbits & 0x1F );
+        }
+    } else {
+        # PA0
+        my $reglsbits = $dbm + 18;
+        $powerlevel = RF69_PALEVEL_PA0_ON | ( $reglsbits & 0x1F );
+    }
+    
+    $self->reset();
+    
+    $self->configure( [ @{ $self->fsk_config }, [ RF69_REG_PALEVEL, $powerlevel ] ]);
+    
+    return $powerlevel;
+}
+
+sub transmit_mw {
+    my $self = shift;
+    my $mwvalue = 10.0 ** ($self->transmit_dbm / 10.0);
+    return sprintf('%.3f', $mwvalue);
+}
+
+sub dump_transmit_power_info {
+    my $self = shift;
+    my $regpalevel = $self->read_register(RF69_REG_PALEVEL);
+    
+    my @dumplines = ();
+    
+    my $amplifiers = $regpalevel >> 5;
+    my $dbmbase    = $regpalevel & 0x1F;
+    my $dbm;
+    if ( $amplifiers == 0b100) {
+        push @dumplines, q(Amplifier : PAO on RFIO);
+        $dbm = $dbmbase - 18;
+    } elsif( $amplifiers == 0b010 ) {
+        push @dumplines, q(Amplifier : PA1 on PA_BOOST pin);
+        $dbm = $dbmbase - 18;
+    } else {
+        push @dumplines, q(Amplifier : PA1 and PA2 on PA_BOOST pin);
+        if ( $self->pa_boost_on) {
+            $dbm = $dbmbase - 11;
+        } else {
+            $dbm = $dbmbase - 14;
+        }
+    }
+    push @dumplines, qq(dBm       : $dbm);
+    push @dumplines, sprintf(qq(PA Boost  : %s), ( $self->pa_boost_on ) ? q(Yes) : q(No) );
+    push @dumplines, sprintf(qq(mw        : %.3f), 10.0 ** ($dbm / 10.0));
+    return ( wantarray )
+        ? @dumplines
+        : join(qq(\n), @dumplines) . qq(\n);
+}
+
 sub configure {
     my( $self, $config ) = @_;
     for my $msgref ( @$config ) {
         $self->write_register(@$msgref);
     }
-    
-    if( $self->high_power_module ) {
-        $self->write_register( RF69_REG_OCP, RF69_VAL_OCP_OFF );
-        $self->write_register( RF69_REG_PALEVEL, ( $self->read_register(RF69_REG_PALEVEL) & 0x1F) | RF69_PALEVEL_PA1_ON | RF69_PALEVEL_PA2_ON );
-    }
-    
     $self->wait_for(RF69_REG_IRQFLAGS1, RF69_MASK_MODEREADY, RF69_TRUE);
 }
 
@@ -137,15 +237,26 @@ sub change_mode {
     my($self, $mode, $waitmask) = @_;
     $waitmask //= RF69_MASK_MODEREADY;
     $self->write_register(RF69_REG_OPMODE, $mode);
-    if( $self->high_power_module && $self->max_power_on ) {
-        if( $mode == RF69_MASK_OPMODE_RX ) {
-            $self->write_register( RF69_REG_TESTPA1, 0x55 );
-            $self->write_register( RF69_REG_TESTPA2, 0x70 );
-        } elsif( $mode == RF69_MASK_OPMODE_TX ) {
+    
+    # high power module with PA_BOOST on 
+    if ( $self->pa_boost_on ) {
+        if( $mode == RF69_MASK_OPMODE_TX ) {
+            # OCP off for transmit
+            my $regocp = $self->read_register(RF69_REG_OCP);
+            $self->write_register( RF69_REG_OCP, $regocp & 0xF );
+            # PA BOOST on for transmit
             $self->write_register( RF69_REG_TESTPA1, 0x5D );
             $self->write_register( RF69_REG_TESTPA2, 0x7C );
+        } else {
+            # OCP on for receive
+            my $regocp = $self->read_register(RF69_REG_OCP);
+            $self->write_register( RF69_REG_OCP, 0x10 | ( $regocp & 0xF ) );
+            # PA BOOST off for receive
+            $self->write_register( RF69_REG_TESTPA1, 0x55 );
+            $self->write_register( RF69_REG_TESTPA2, 0x70 );
         }
     }
+    
     $self->wait_for(RF69_REG_IRQFLAGS1, $waitmask, RF69_TRUE);
 }
 
@@ -319,6 +430,25 @@ sub receive_message {
     return undef;
 }
 
+sub send_hipi_message {
+    my ($self, $msg) =  @_;
+    $msg->encode_buffer unless $msg->is_encoded;
+    $self->send_message( $msg->databuffer );
+}
+
+sub receive_hipi_message {
+    my ($self, $messageclass, $messageparams ) = @_;
+    
+    if( my $buffer = $self->receive_message ) {
+        $messageparams->{databuffer} = $buffer;
+        my $msg = $messageclass->new(
+            %$messageparams
+        );
+        $msg->inspect_buffer;
+        return $msg;
+    }
+    return undef;
+}
 
 1;
 
