@@ -14,7 +14,7 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION ='0.86';
+our $VERSION ='0.89';
 
 my ( $btype1, $btype2, $btype3, $btype4) = ( 1, 2, 3, 4 );
 
@@ -22,6 +22,7 @@ my $israspberry = 0;
 my $israspberry2 = 0;
 my $israspberry3 = 0;
 my $israspberry4 = 0;
+my $israspberry5 = 0;
 my $hasdevicetree = 0;
 my $homedir = '/tmp';
 
@@ -149,6 +150,71 @@ my @_alt_func_names_2711 =
     [ 'SD0_DAT3'        , 'PWM0_1'          , 'PCM_DOUT'        , 'SD1_DAT3'        , 'ARM_TMS'         , '-'               ], # 53
 );
 
+my @_alt_func_names_2712 =
+(
+    # BANK 0
+    #  spi0,           dpi,         uart1,          i2c0,           _,             gpio,           proc_rio,       pio,          spi2),
+    [ 'SPI0_SIO[3]',  'DPI_PCLK',  'UART1_TX',     'I2C0_SDA',     '-',           'SYS_RIO[0]',   'PROC_RIO[0]',  'PIO[0]',      'SPI2_CSn[0]' ], #  0
+    [ 'SPI0_SIO[2]',  'DPI_DE',    'UART1_RX',     'I2C0_SCL',     '-',           'SYS_RIO[1]',   'PROC_RIO[1]',  'PIO[1]',      'SPI2_SIO[1]' ], #  1
+    [ 'SPI0_CSn[3]',  'DPI_VSYNC', 'UART1_CTS',    'I2C1_SDA',     'UART0_IR_RX', 'SYS_RIO[2]',   'PROC_RIO[2]',  'PIO[2]',      'SPI2_SIO[0]' ], #  2
+    [ 'SPI0_CSn[2]',  'DPI_HSYNC', 'UART1_RTS',    'I2C1_SCL',     'UART0_IR_TX', 'SYS_RIO[3]',   'PROC_RIO[3]',  'PIO[3]',      'SPI2_SCLK'   ], #  3
+    [ 'GPCLK[0]',     'DPI_D[0]',  'UART2_TX',     'I2C2_SDA',     'UART0_RI',    'SYS_RIO[4]',   'PROC_RIO[4]',  'PIO[4]',      'SPI3_CSn[0]' ], #  4
+    [ 'GPCLK[1]',     'DPI_D[1]',  'UART2_RX',     'I2C2_SCL',     'UART0_DTR',   'SYS_RIO[5]',   'PROC_RIO[5]',  'PIO[5]',      'SPI3_SIO[1]' ], #  5
+    [ 'GPCLK[2]',     'DPI_D[2]',  'UART2_CTS',    'I2C3_SDA',     'UART0_DCD',   'SYS_RIO[6]',   'PROC_RIO[6]',  'PIO[6]',      'SPI3_SIO[0]' ], #  6
+    [ 'SPI0_CSn[1]',  'DPI_D[3]',  'UART2_RTS',    'I2C3_SCL',     'UART0_DSR',   'SYS_RIO[7]',   'PROC_RIO[7]',  'PIO[7]',      'SPI3_SCLK'   ], #  7
+    [ 'SPI0_CSn[0]',  'DPI_D[4]',  'UART3_TX',     'I2C0_SDA',     '-',           'SYS_RIO[8]',   'PROC_RIO[8]',  'PIO[8]',      'SPI4_CSn[0]' ], #  8  
+    [ 'SPI0_SIO[1]',  'DPI_D[5]',  'UART3_RX',     'I2C0_SCL',     '-',           'SYS_RIO[9]',   'PROC_RIO[9]',  'PIO[9]',      'SPI4_SIO[0]' ], #  9
+    [ 'SPI0_SIO[0]',  'DPI_D[6]',  'UART3_CTS',    'I2C1_SDA',     '-',           'SYS_RIO[10]',  'PROC_RIO[10]', 'PIO[10]',     'SPI4_SIO[1]' ], # 10
+    [ 'SPI0_SCLK',    'DPI_D[7]',  'UART3_RTS',    'I2C1_SCL',     '-',           'SYS_RIO[11]',  'PROC_RIO[11]', 'PIO[11]',     'SPI4_SCLK',  ], # 11
+    [ 'PWM0[0]',      'DPI_D[8]',  'UART4_TX',     'I2C2_SDA',     'AUDIO_OUT_L', 'SYS_RIO[12]',  'PROC_RIO[12]', 'PIO[12]',     'SPI5_CSn[0]' ], # 12
+    [ 'PWM0[1]',      'DPI_D[9]',  'UART4_RX',     'I2C2_SCL',     'AUDIO_OUT_R', 'SYS_RIO[13]',  'PROC_RIO[13]', 'PIO[13]',     'SPI5_SIO[1]' ], # 13
+    [ 'PWM0[2]',      'DPI_D[10]', 'UART4_CTS',    'I2C3_SDA',     'UART0_TX',    'SYS_RIO[14]',  'PROC_RIO[14]', 'PIO[14]',     'SPI5_SIO[0]' ], # 14
+    [ 'PWM0[3]',      'DPI_D[11]', 'UART4_RTS',    'I2C3_SCL',     'UART0_RX',    'SYS_RIO[15]',  'PROC_RIO[15]', 'PIO[15]',     'SPI5_SCLK'   ], # 15
+    [ 'SPI1_CSn[2]',  'DPI_D[12]', 'MIPI0_DSI_TE', '-',            'UART0_CTS',   'SYS_RIO[16]',  'PROC_RIO[16]', 'PIO[16]',      '-'          ], # 16
+    [ 'SPI1_CSn[1]',  'DPI_D[13]', 'MIPI1_DSI_TE', '-',            'UART0_RTS',   'SYS_RIO[17]',  'PROC_RIO[17]', 'PIO[17]',      '-'          ], # 17
+    [ 'SPI1_CSn[0]',  'DPI_D[14]', 'I2S0_SCLK',    'PWM0[2]',      'I2S1_SCLK',   'SYS_RIO[18]',  'PROC_RIO[18]', 'PIO[18]',     'GPCLK[1]'    ], # 18
+    [ 'SPI1_SIO[1]',  'DPI_D[15]', 'I2S0_WS',      'PWM0[3]',      'I2S1_WS',     'SYS_RIO[19]',  'PROC_RIO[19]', 'PIO[19]',     '-'           ], # 19
+    [ 'SPI1_SIO[0]',  'DPI_D[16]', 'I2S0_SDI[0]',  'GPCLK[0]',     'I2S1_SDI[0]', 'SYS_RIO[20]',  'PROC_RIO[20]', 'PIO[20]',     '-'           ], # 20
+    [ 'SPI1_SCLK',    'DPI_D[17]', 'I2S0_SDO[0]',  'GPCLK[1]',     'I2S1_SDO[0]', 'SYS_RIO[21]',  'PROC_RIO[21]', 'PIO[21]',     '-'           ], # 21
+    [ 'SDIO0_CLK',    'DPI_D[18]', 'I2S0_SDI[1]',  'I2C3_SDA',     'I2S1_SDI[1]', 'SYS_RIO[22]',  'PROC_RIO[22]', 'PIO[22]',     '-'           ], # 22
+    [ 'SDIO0_CMD',    'DPI_D[19]', 'I2S0_SDO[1]',  'I2C3_SCL',     'I2S1_SDO[1]', 'SYS_RIO[23]',  'PROC_RIO[23]', 'PIO[23]',     '-'           ], # 23
+    [ 'SDIO0_DAT[0]', 'DPI_D[20]', 'I2S0_SDI[2]',  '-',            'I2S1_SDI[2]', 'SYS_RIO[24]',  'PROC_RIO[24]', 'PIO[24]',     'SPI2_CSn[1]' ], # 24
+    [ 'SDIO0_DAT[1]', 'DPI_D[21]', 'I2S0_SDO[2]',  'AUDIO_IN_CLK', 'I2S1_SDO[2]', 'SYS_RIO[25]',  'PROC_RIO[25]', 'PIO[25]',     'SPI3_CSn[1]' ], # 25
+    [ 'SDIO0_DAT[2]', 'DPI_D[22]', 'I2S0_SDI[3]',  'AUDIO_IN_DAT', 'I2S1_SDI[3]', 'SYS_RIO[26]',  'PROC_RIO[26]', 'PIO[26]',     'SPI5_CSn[1]' ], # 26
+    [ 'SDIO0_DAT[3]', 'DPI_D[23]', 'I2S0_SDO[3]',  'AUDIO_IN_DAT', 'I2S1_SDO[3]', 'SYS_RIO[27]',  'PROC_RIO[27]', 'PIO[27]',     'SPI1_CSn[1]' ], # 27
+    
+    # BANK 1
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 28
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 29
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 30
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 31
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 32
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 33
+   
+    # BANK 2
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 34
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 35
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 36
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 37
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 38
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 39
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 40
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 41
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 42
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 43
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 44
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 45
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 46
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 47
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 48
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 49
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 50
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 51
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 52
+    [ '-',            '-',         '-',            '-',            '-',           '-',            '-',            '-',           '-'           ], # 53
+    
+);
+
 my $_alt_function_names;
 my $_alt_function_names_version;
 
@@ -196,14 +262,45 @@ my %_revinfostash = (
         '4' => 'Embest',
         '5' => 'Stadium',
     },
-    
     processor => {
         '0' => 'BCM2835',
         '1' => 'BCM2836',
         '2' => 'BCM2837',
         '3' => 'BCM2711',
+        '4' => 'BCM2712',
+    },
+    processor_info => {
+        'BCM2835' => {
+            arm                 => 'ARM1176',
+            cores               => 1,
+            architecture_width  => 32,
+            raspios_supported   => [ 'armhf' ],
+        },
+        'BCM2836' => {
+            arm                 => 'Cortex-A7',
+            cores               => 4,
+            architecture_width  => 32,
+            raspios_supported   => [ 'armhf' ],
+        },
+        'BCM2837' => {
+            arm                 => 'Cortex-A53',
+            cores               => 4,
+            architecture_width  => 64,
+            raspios_supported   => [ 'armhf', 'arm64' ],
+        },
+        'BCM2711' => {
+            arm                 => 'Cortex-A72',
+            cores               => 4,
+            architecture_width  => 64,
+            raspios_supported   => [ 'armhf', 'arm64' ],
+        },
+        'BCM2712' => {
+            arm                 => 'Cortex-A76',
+            cores               => 4,
+            architecture_width  => 64,
+            raspios_supported   => [ 'armhf', 'arm64' ],
+        },
     },
-    
     type => {
         '0'  => 'Raspberry Pi Model A',                 # 00
         '1'  => 'Raspberry Pi Model B',                 # 01
@@ -223,9 +320,12 @@ my %_revinfostash = (
         '15' => 'UNKNOWN Rasberry Pi Model 15',         # 0F
         '16' => 'Raspberry Pi Compute Module 3 Plus',   # 10
         '17' => 'Raspberry Pi 4 Model B',               # 11
-        '18' => 'Raspberry Pi Zero 2',                  # 12
+        '18' => 'Raspberry Pi Zero 2 W',                # 12
         '19' => 'Raspberry Pi Model 400',               # 13
         '20' => 'Raspberry Pi Compute Module 4',        # 14
+        '21' => 'Raspberry Pi Compute Module 4S',       # 15
+        '22' => 'UNKOWN Rasberry Pi Model 22',          # 16
+        '23' => 'Rasberry Pi 5',                        # 17
     },
     board_type => {
         '0'  => $btype2,
@@ -249,6 +349,9 @@ my %_revinfostash = (
         '18' => $btype3,
         '19' => $btype3,
         '20' => $btype4,
+        '21' => $btype4,
+        '22' => $btype3,
+        '23' => $btype3,
     },
     release => {
         '0'  => 'Q1 2013',
@@ -272,14 +375,25 @@ my %_revinfostash = (
         '18' => 'Q4 2021',
         '19' => 'Q4 2020',
         '20' => 'Q4 2020',
+        '21' => 'Q4 2020',
+        '22' => 'unknown',
+        '23' => 'Q4 2023',
     },
     extended_release => {
         'a03111' => 'Q2 2019', #	4B	1.1	1GB	Sony UK
+        
         'b03111' => 'Q2 2019', #	4B	1.1	2GB	Sony UK
         'b03112' => 'Q1 2020', #	4B	1.2	2GB	Sony UK
+        'b03114' => 'Q3 2020', #	4B	1.4	2GB	Sony UK
+        'b03115' => 'Q1 2022', #	4B	1.5	2GB	Sony UK
+        
         'c03111' => 'Q2 2019', # 	4B	1.1	4GB	Sony UK
         'c03112' => 'Q1 2020', #	4B	1.2	4GB	Sony UK
+        'c03114' => 'Q2 2020', #	4B	1.4	4GB	Sony UK
+        'c03115' => 'Q1 2022', #	4B	1.5	4GB	Sony UK
+        
         'd03114' => 'Q2 2020', #	4B	1.4	8GB	Sony UK
+        'd03115' => 'Q1 2022', #	4B	1.5	8GB	Sony UK
     },
 );
 
@@ -303,6 +417,8 @@ sub is_raspberry_3 { return $israspberry3; }
 
 sub is_raspberry_4 { return $israspberry4; }
 
+sub is_raspberry_5 { return $israspberry5; }
+
 sub has_device_tree { return $hasdevicetree; }
 
 sub home_directory { return $homedir; }
@@ -317,6 +433,8 @@ sub release_date { return $_config->{release}; }
 
 sub processor { return $_config->{processor}; }
 
+sub has_rp1 { return ( $israspberry5 ) ? 1 : 0; }
+
 sub hardware { return $_config->{hardware}; }
 
 sub model_name { return $_config->{modelname}; }
@@ -327,10 +445,24 @@ sub memory { return $_config->{memory}; }
 
 sub serial_number { return $_config->{serial}; }
 
+sub short_serial_number { return $_config->{short_serial}; }
+
 sub get_alt_function_names { return $_alt_function_names; }
 
 sub alt_func_version { return $_alt_function_names_version; }
 
+sub architecture_width { return $_config->{processor_info}->{architecture_width}; }
+
+sub arm_core { return $_config->{processor_info}->{arm}; }
+
+sub rasberrypi_os_support {
+    return ( wantarray )
+        ? @{ $_config->{processor_info}->{raspios_supported} }
+        : join(', ', @{ $_config->{processor_info}->{raspios_supported} } );
+}
+
+sub number_of_cores { return $_config->{processor_info}->{cores}; }
+
 sub board_description {
     my $description = 'Unknown board type';
     if($_config->{board_type} == $btype1 ) {
@@ -374,7 +506,9 @@ sub _configure {
     }
     
     my $hardware = ($_cpuinfostash{Hardware}) ?  $_cpuinfostash{Hardware} : 'BCM2709';
-    my $serial = ($_cpuinfostash{Serial}) ?  $_cpuinfostash{Serial} : 'UNKNOWN';
+    my $serial = ($_cpuinfostash{Serial}) ?  $_cpuinfostash{Serial} : 'SERIALNONOTFOUND';
+    my $short_serial = $serial;
+    $short_serial =~ s/^(.{8})(.{8})$/$2/;
     my $board_type = ( $hardware eq 'BCM2708' ) ? 2 : 3;
     my $defaultkey = ( $board_type == 3  ) ? 'unknownex' : 'unknown';
     my $rev = ($_cpuinfostash{Revision}) ?  lc( $_cpuinfostash{Revision} ) : $defaultkey;
@@ -392,43 +526,64 @@ sub _configure {
         # is this a scheme 0 or 1 number
         my $revnum = oct( '0x' . $rev );
         #          NOQuuuWuFMMMCCCCPPPPTTTTTTTTRRRR
-        my $schemenewt = 0b100000000000000000000000 & $revnum;
-        $schemenewt = $schemenewt >> 23;
+        #                  ^
+        my $rev_scheme_new_type = ( $revnum >> 23 ) & 1;
         
-        if ( $schemenewt ) {
-            # revision         RRRR
-            my $schemerev  = 0b1111 & $revnum;
-            # raspberry type     TTTTTTTTRRRR
-            my $schemetype = ( 0b111111110000 & $revnum ) >> 4;
-            # processor          PPPPTTTTTTTTRRRR
-            my $schemeproc = ( 0b1111000000000000 & $revnum) >> 12;
-            # manufacturer       CCCCPPPPTTTTTTTTRRRR
-            my $schememanu = ( 0b11110000000000000000 & $revnum ) >> 16;
-            # memory             MMMCCCCPPPPTTTTTTTTRRRR
-            my $schemesize = ( 0b11100000000000000000000 & $revnum ) >> 20;           
-            # other bits         NOQuuuWuFMMMCCCCPPPPTTTTTTTTRRRR
-            # my $schemerest = 0b11100010000000000000000000000000 & $revnum;
-            my $schemewarranty = ( $revnum >> 25 ) & 1;
-            my $schemeotpread  = ( $revnum >> 29 ) & 1;
-            my $schemeotpprog  = ( $revnum >> 30 ) & 1;
-            my $schemevolt     = ( $revnum >> 31 ) & 1;
+        if ( $rev_scheme_new_type ) {
+            #            NOQuuuWuFMMMCCCCPPPPTTTTTTTTRRRR
+
+            # revision                               RRRR
+            my $s_revision  =                      0b1111 & $revnum;
+            # raspberry type                 TTTTTTTT
+            my $s_raspberry_type =       ( 0b111111110000 & $revnum ) >> 4;
+            # processor                  PPPP
+            my $s_processor =        ( 0b1111000000000000 & $revnum ) >> 12;
+            # manufacturer           CCCC
+            my $s_manufacturer = ( 0b11110000000000000000 & $revnum ) >> 16;
+            # memory              MMM
+            my $s_memory =    ( 0b11100000000000000000000 & $revnum ) >> 20;           
+            # warranty   NOQuuuWuFMMMCCCCPPPPTTTTTTTTRRRR
+            #                  ^
+            my $s_warranty    = ( $revnum >> 25 ) & 1;
+            # otp read   NOQuuuWuFMMMCCCCPPPPTTTTTTTTRRRR
+            #              ^
+            my $s_otp_read    = ( $revnum >> 29 ) & 1;
+            # otp prog   NOQuuuWuFMMMCCCCPPPPTTTTTTTTRRRR
+            #             ^
+            my $s_otp_prog    = ( $revnum >> 30 ) & 1;
+            # no o volt  NOQuuuWuFMMMCCCCPPPPTTTTTTTTRRRR
+            #            ^
+            my $s_no_overvolt = ( $revnum >> 31 ) & 1;
             
             # base type
             my $binfo = $_revstash{$defaultkey};
                         
-            $binfo->{release}  = $_revinfostash{extended_release}->{$rev} || $_revinfostash{release}->{$schemetype} || 'Q1 2015';
-            $binfo->{model_name} = $_revinfostash{type}->{$schemetype} || qq(Unknown Raspberry Pi Type : $schemetype);
+            $binfo->{release}  = $_revinfostash{extended_release}->{$rev} || $_revinfostash{release}->{$s_raspberry_type} || 'Q1 2015';
+            $binfo->{model_name} = $_revinfostash{type}->{$s_raspberry_type} || qq(Unknown Raspberry Pi Type : $s_raspberry_type);
             $binfo->{model_name} = $device_tree_boardname if $device_tree_boardname;
-            $binfo->{memory}   = $_revinfostash{memsize}->{$schemesize} || 256;
-            $binfo->{manufacturer} = $_revinfostash{manufacturer}->{$schememanu} || 'Sony';
-            $binfo->{board_type} =  $_revinfostash{board_type}->{$schemetype} || $board_type;
-            $binfo->{processor} = $_revinfostash{processor}->{$schemeproc} || 'BCM2835';
+            $binfo->{memory}   = $_revinfostash{memsize}->{$s_memory} || 256;
+            $binfo->{manufacturer} = $_revinfostash{manufacturer}->{$s_manufacturer} || 'Sony';
+            $binfo->{board_type} =  $_revinfostash{board_type}->{$s_raspberry_type} || $board_type;
+            $binfo->{processor} = $_revinfostash{processor}->{$s_processor} || 'BCM2835';
             $binfo->{revision} = $rev;
-            $binfo->{revisionnumber} = $schemerev;
+            $binfo->{revisionnumber} = $s_revision;
+            $binfo->{processor_info} = $_revinfostash{processor_info}->{$binfo->{processor}};
             
-            $israspberry2 = ( $schemetype == 4 ) ? 1 : 0;
-            $israspberry3 = ( $schemetype == 8 || $schemetype == 10 || $schemetype == 13 || $schemetype == 14 || $schemetype == 16 || $schemetype == 18 ) ? 1 : 0;
-            $israspberry4 = ( $schemetype == 17 ) ? 1 : 0;
+            $israspberry2 = ( $s_raspberry_type == 4 ) ? 1 : 0;
+            
+            $israspberry3 = ( $s_raspberry_type == 8  ||
+                              $s_raspberry_type == 10 ||
+                              $s_raspberry_type == 13 ||
+                              $s_raspberry_type == 14 ||
+                              $s_raspberry_type == 16 ||
+                              $s_raspberry_type == 18 ) ? 1 : 0;
+            
+            $israspberry4 = ( $s_raspberry_type == 17 ||
+                              $s_raspberry_type == 19 ||
+                              $s_raspberry_type == 20 ||
+                              $s_raspberry_type == 21 ) ? 1 : 0;
+            
+            $israspberry5 = ( $s_raspberry_type == 23 ) ? 1 : 0;
             
             $_config = { %$binfo };
         } else {
@@ -452,8 +607,12 @@ sub _configure {
     
     $_config->{hardware} = $hardware;
     $_config->{serial}  = $serial;
+    $_config->{short_serial}  = $short_serial;
     
-    if($_config->{processor} eq 'BCM2711' ) {
+    if($_config->{processor} eq 'BCM2712' ) {
+        $_alt_function_names = \@_alt_func_names_2712;
+        $_alt_function_names_version = 3;
+    } elsif($_config->{processor} eq 'BCM2711' ) {
         $_alt_function_names = \@_alt_func_names_2711;
         $_alt_function_names_version = 2;
     } else {
@@ -464,8 +623,6 @@ sub _configure {
     return;
 }
 
-
-
 sub new {
     my $class = shift;
     my $self = bless {}, $class;
@@ -485,6 +642,7 @@ sub validpins {
 }
 
 sub dump_board_info {
+    my $processor = $_config->{processor};
     my $dump = qq(--------------------------------------------------\n);
     $dump .= qq(Raspberry Pi Board Info\n);
     $dump .= qq(--------------------------------------------------\n);
@@ -492,12 +650,13 @@ sub dump_board_info {
     $dump .= qq(Released         : $_config->{release}\n);
     $dump .= qq(Manufacturer     : $_config->{manufacturer}\n);
     $dump .= qq(Memory           : $_config->{memory}\n);
-    $dump .= qq(Processor        : $_config->{processor}\n);
+    $dump .= qq(Processor        : $processor\n);
     $dump .= qq(Hardware         : $_config->{hardware}\n);
     my $description = board_description();
     $dump .= qq(Description      : $description\n);
     $dump .= qq(Revision         : $_config->{revision}\n);
     $dump .= qq(Serial Number    : $_config->{serial}\n);
+    $dump .= qq(Short Serial No  : $_config->{short_serial}\n);
     $dump .= qq(GPIO Header Type : $_config->{board_type}\n);
     $dump .= qq(Revision Number  : $_config->{revisionnumber}\n);
     my $devtree = ( has_device_tree() ) ? 'Yes' : 'No';
@@ -507,9 +666,15 @@ sub dump_board_info {
     $dump .= q(Is Raspberry 2   : ) . (($israspberry2) ? 'Yes' : 'No' ) . qq(\n);
     $dump .= q(Is Raspberry 3   : ) . (($israspberry3) ? 'Yes' : 'No' ) . qq(\n);
     $dump .= q(Is Raspberry 4   : ) . (($israspberry4) ? 'Yes' : 'No' ) . qq(\n);
+    $dump .= q(Is Raspberry 5   : ) . (($israspberry5) ? 'Yes' : 'No' ) . qq(\n);
     
     $dump .= q(Alt Function Map : Version ) . alt_func_version() . qq(\n);
     
+    $dump .= qq(ARM Core         : ) . arm_core() . qq(\n);
+    $dump .= qq(Number of Cores  : ) . number_of_cores() . qq(\n);
+    $dump .= qq(Architecture     : Width ) . architecture_width() . qq( bit\n);
+    $dump .= qq(OS arch support  : ) . rasberrypi_os_support() . qq(\n);
+    
     return $dump;
 }
 
