@@ -1,7 +1,7 @@
 #########################################################################################
 # Package        HiPi::Utils::Exec
 # Description  : Executable Wrappers
-# Copyright    : Copyright (c) 2013-2017 Mark Dootson
+# Copyright    : Copyright (c) 2013-2023 Mark Dootson
 # License      : This is free software; you can redistribute it and/or modify it under
 #                the same terms as the Perl 5 programming language system itself.
 #########################################################################################
@@ -23,7 +23,7 @@ use HiPi;
 
 __PACKAGE__->create_accessors( qw( workingdir sourceperl outputexec ) );
 
-our $VERSION ='0.81';
+our $VERSION ='0.89';
 
 XSLoader::load('HiPi::Utils::Exec', $VERSION) if HiPi::is_raspberry_pi();
 
@@ -186,8 +186,7 @@ PEPIMAKEFILETEMPLATE
     #$libpaths =~ s/\s+/ -L/g;
     my $ldflags = $optimise . ' ' . $Config{ldflags} ;
     $template =~ s/REPLACELDFLAGS/$ldflags/g;
-    
-    my $ldlibs = '-ldl -lm -lpthread -lc -lcrypt -lz -lperl';
+    my $ldlibs = $Config{perllibs} . ' -lz -lperl';
     $template =~ s/REPLACELDLIBS/$ldlibs/g;
     
     return $template;
